package com.supervision.pdfqaserver.dto;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

/**
 * CypherSchemaDTO
 */
public class CypherSchemaDTO {

    private List<EntityExtractionDTO> nodes = new ArrayList<>();

    private List<RelationExtractionDTO> relations = new ArrayList<>();

    public CypherSchemaDTO(List<EntityExtractionDTO> nodes, List<RelationExtractionDTO> relations) {
        this.nodes = nodes;
        this.relations = relations;
    }

    /**
     * 根据头节点、尾节点、关系获取关系抽取DTO
     * @param sourceType 源节点类型
     * @param relation 关系
     * @param targetType 尾节点类型
     * @return
     */
    public RelationExtractionDTO getRelation(String sourceType, String relation,String targetType) {
        for (RelationExtractionDTO relationDTO : relations) {
            if (StrUtil.equals(relationDTO.getSourceType(), sourceType) &&
                StrUtil.equals(relationDTO.getRelation(), relation) &&
                StrUtil.equals(relationDTO.getTargetType(), targetType)) {
                return relationDTO;
            }
        }
        return null;
    }

    /**
     * 根据源节点类型或目标节点类型获取关系抽取DTO列表
     * @param sourceOrTargetType
     * @return
     */
    public List<RelationExtractionDTO> getRelations(String sourceOrTargetType) {
        List<RelationExtractionDTO> result = new ArrayList<>();
        for (RelationExtractionDTO relationDTO : relations) {
            if (StrUtil.equals(relationDTO.getSourceType(), sourceOrTargetType) ||
                StrUtil.equals(relationDTO.getTargetType(), sourceOrTargetType)) {
                result.add(relationDTO);
            }
        }
        return result;
    }

    public List<EntityExtractionDTO> getNodes() {
        return nodes;
    }

    public List<RelationExtractionDTO> getRelations() {
        return relations;
    }

    /**
     * 根据实体名获取关系抽取DTO列表
     * @param entity
     * @return
     */
    public EntityExtractionDTO getNode(String entity) {
        for (EntityExtractionDTO node : nodes) {
            if (StrUtil.equals(node.getEntity(), entity)) {
                return node;
            }
        }
        return null;
    }

    public String format(){
        JSONObject nodeJson = new JSONObject();
        for (EntityExtractionDTO node : nodes) {
            String entity = node.getEntity();
            List<TruncationERAttributeDTO> attributes = node.getAttributes();
            JSONObject nodeAttr = nodeJson.getJSONObject(entity);
            if (nodeAttr == null) {
                nodeAttr = new JSONObject();
                nodeJson.set(entity, nodeAttr);
            }
            for (TruncationERAttributeDTO attribute : attributes) {
                boolean none = nodeAttr.entrySet().stream().noneMatch(
                        entry -> StrUtil.equals(entry.getKey(), attribute.getAttribute()));
                if (none){
                    nodeAttr.set(attribute.getAttribute(), attribute.getDataType());
                }
            }

        }
        JSONObject relJson = new JSONObject();
        for (RelationExtractionDTO relation : relations) {
            String sourceType = relation.getSourceType();
            String targetType = relation.getTargetType();
            String rela = relation.getRelation();
            JSONObject json = relJson.getJSONObject(rela);
            if (null == json) {
                json = new JSONObject();
                relJson.set(rela, json);
            }
            json.set("_endpoints", new JSONArray(new String[]{sourceType, targetType}));
            for (TruncationERAttributeDTO attribute : relation.getAttributes()) {
                boolean none = json.entrySet().stream().noneMatch(
                        entry -> StrUtil.equals(entry.getKey(), attribute.getAttribute())
                );
                if (none) {
                    json.set(attribute.getAttribute(), attribute.getDataType());
                }
            }
        }
        JSONObject object = new JSONObject()
                .set("nodetypes", nodeJson)
                .set("relationshiptypes", relJson);
        return object.toString();

    }
}
